from __future__ import absolute_import, unicode_literals

from documents.permissions import permission_document_view
from documents.tests.test_views import GenericDocumentViewTestCase
from user_management.tests import (
    TEST_USER_USERNAME, TEST_USER_PASSWORD
)

from ..models import Folder
from ..permissions import (
    permission_folder_add_document, permission_folder_create,
    permission_folder_delete, permission_folder_edit,
    permission_folder_remove_document, permission_folder_view
)
from .literals import TEST_FOLDER_LABEL, TEST_FOLDER_EDITED_LABEL


class FolderViewTestCase(GenericDocumentViewTestCase):
    def test_folder_create_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        response = self.post(
            'folders:folder_create', data={
                'label': TEST_FOLDER_LABEL
            }
        )

        self.assertEquals(response.status_code, 403)
        self.assertEqual(Folder.objects.count(), 0)

    def test_folder_create_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_folder_create.stored_permission
        )

        response = self.post(
            'folders:folder_create', data={
                'label': TEST_FOLDER_LABEL
            }, follow=True
        )
        self.assertContains(response, text='created', status_code=200)
        self.assertEqual(Folder.objects.count(), 1)
        self.assertEqual(Folder.objects.first().label, TEST_FOLDER_LABEL)

    def test_folder_create_duplicate_view_with_permission(self):
        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_folder_create.stored_permission
        )

        response = self.post(
            'folders:folder_create', data={
                'label': TEST_FOLDER_LABEL
            }
        )

        self.assertContains(response, text='exists', status_code=200)
        self.assertEqual(Folder.objects.count(), 1)
        self.assertEqual(Folder.objects.first().pk, folder.pk)

    def test_folder_delete_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post('folders:folder_delete', args=(folder.pk,))
        self.assertEqual(response.status_code, 403)
        self.assertEqual(Folder.objects.count(), 1)

    def test_folder_delete_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_folder_delete.stored_permission
        )

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_delete', args=(folder.pk,), follow=True
        )

        self.assertContains(response, text='deleted', status_code=200)
        self.assertEqual(Folder.objects.count(), 0)

    def test_folder_edit_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_edit', args=(folder.pk,), data={
                'label': TEST_FOLDER_EDITED_LABEL
            }
        )
        self.assertEqual(response.status_code, 403)
        folder = Folder.objects.get(pk=folder.pk)
        self.assertEqual(folder.label, TEST_FOLDER_LABEL)

    def test_folder_edit_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_folder_edit.stored_permission
        )

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_edit', args=(folder.pk,), data={
                'label': TEST_FOLDER_EDITED_LABEL
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='update', status_code=200)
        self.assertEqual(folder.label, TEST_FOLDER_EDITED_LABEL)

    def test_folder_add_document_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_folder_view.stored_permission)

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_add_document', args=(self.document.pk,), data={
                'folder': folder.pk,
            }
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(folder.documents.count(), 0)

    def test_folder_add_document_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_folder_view.stored_permission)

        self.role.permissions.add(
            permission_folder_add_document.stored_permission
        )

        self.role.permissions.add(
            permission_document_view.stored_permission
        )

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_add_document', args=(self.document.pk,), data={
                'folder': folder.pk,
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='added', status_code=200)
        self.assertEqual(folder.documents.count(), 1)
        self.assertQuerysetEqual(
            folder.documents.all(), (repr(self.document),)
        )

    def test_folder_add_multiple_documents_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_folder_view.stored_permission)

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_add_multiple_documents', data={
                'id_list': (self.document.pk,), 'folder': folder.pk
            }
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(folder.documents.count(), 0)

    def test_folder_add_multiple_documents_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_folder_view.stored_permission)

        self.role.permissions.add(
            permission_folder_add_document.stored_permission
        )

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        response = self.post(
            'folders:folder_add_multiple_documents', data={
                'id_list': (self.document.pk,), 'folder': folder.pk
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='added', status_code=200)
        self.assertEqual(folder.documents.count(), 1)
        self.assertQuerysetEqual(
            folder.documents.all(), (repr(self.document),)
        )

    def test_folder_remove_document_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        folder.documents.add(self.document)

        self.assertEqual(folder.documents.count(), 1)

        response = self.post(
            'folders:folder_document_multiple_remove', args=(folder.pk,),
            data={
                'id_list': (self.document.pk,),
            }
        )

        self.assertEqual(response.status_code, 302)

        folder = Folder.objects.get(pk=folder.pk)
        self.assertEqual(folder.documents.count(), 1)

    def test_folder_remove_document_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_folder_remove_document.stored_permission
        )

        folder = Folder.objects.create(label=TEST_FOLDER_LABEL)

        folder.documents.add(self.document)
        self.assertEqual(folder.documents.count(), 1)

        response = self.post(
            'folders:folder_document_multiple_remove', args=(folder.pk,),
            data={
                'id_list': (self.document.pk,),
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='removed', status_code=200)
        self.assertEqual(folder.documents.count(), 0)



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    ACLCreateView, ACLDeleteView, ACLListView, ACLPermissionsView
)

urlpatterns = patterns(
    'acls.views',
    url(
        r'^(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/create/$',
        ACLCreateView.as_view(), name='acl_create'
    ),
    url(
        r'^(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/list/$',
        ACLListView.as_view(), name='acl_list'
    ),
    url(r'^(?P<pk>\d+)/delete/$', ACLDeleteView.as_view(), name='acl_delete'),
    url(
        r'^(?P<pk>\d+)/permissions/$', ACLPermissionsView.as_view(),
        name='acl_permissions'
    ),
)



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, reverse_lazy
from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _, ungettext

from acls.models import AccessControlList
from common.views import (
    SingleObjectCreateView, SingleObjectEditView, SingleObjectListView
)
from documents.models import Document
from documents.views import DocumentListView
from documents.permissions import permission_document_view
from permissions import Permission

from .forms import TagListForm
from .models import Tag
from .permissions import (
    permission_tag_attach, permission_tag_create, permission_tag_delete,
    permission_tag_edit, permission_tag_remove, permission_tag_view
)

logger = logging.getLogger(__name__)


class TagCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create tag')}
    fields = ('label', 'color')
    model = Tag
    post_action_redirect = reverse_lazy('tags:tag_list')
    view_permission = permission_tag_create


def tag_attach(request, document_id=None, document_id_list=None):
    if document_id:
        queryset = Document.objects.filter(pk=document_id)
        post_action_redirect = reverse('tags:tag_list')
    elif document_id_list:
        queryset = Document.objects.filter(pk__in=document_id_list)

    try:
        Permission.check_permissions(request.user, (permission_tag_attach,))
    except PermissionDenied:
        queryset = AccessControlList.objects.filter_by_access(
            permission_tag_attach, request.user, queryset
        )

    if not queryset:
        if document_id:
            raise PermissionDenied
        else:
            messages.error(request, _('Must provide at least one document.'))
            return HttpResponseRedirect(
                request.META.get(
                    'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
                )
            )

    post_action_redirect = None
    previous = request.POST.get(
        'previous', request.GET.get(
            'previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))
        )
    )
    next = request.POST.get(
        'next', request.GET.get(
            'next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))
        )
    )

    if request.method == 'POST':
        form = TagListForm(request.POST, user=request.user)
        if form.is_valid():
            tag = form.cleaned_data['tag']
            for document in queryset:
                if tag in document.attached_tags().all():
                    messages.warning(
                        request, _(
                            'Document "%(document)s" is already tagged as "%(tag)s"'
                        ) % {
                            'document': document, 'tag': tag
                        }
                    )
                else:
                    tag.documents.add(document)
                    messages.success(
                        request,
                        _(
                            'Tag "%(tag)s" attached successfully to document "%(document)s".'
                        ) % {
                            'document': document, 'tag': tag
                        }
                    )
            return HttpResponseRedirect(next)
    else:
        form = TagListForm(user=request.user)

    context = {
        'form': form,
        'previous': previous,
        'next': next,
        'title': ungettext(
            'Attach tag to document',
            'Attach tag to documents',
            queryset.count()
        )
    }

    if queryset.count() == 1:
        context['object'] = queryset.first()

    return render_to_response(
        'appearance/generic_form.html', context,
        context_instance=RequestContext(request)
    )


def tag_multiple_attach(request):
    return tag_attach(
        request, document_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )


class TagListView(SingleObjectListView):
    object_permission = permission_tag_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'title': _('Tags'),
        }

    def get_queryset(self):
        self.queryset = self.get_tag_queryset()
        return super(TagListView, self).get_queryset()

    def get_tag_queryset(self):
        return Tag.objects.all()


def tag_delete(request, tag_id=None, tag_id_list=None):
    post_action_redirect = None

    if tag_id:
        queryset = Tag.objects.filter(pk=tag_id)
        post_action_redirect = reverse('tags:tag_list')
    elif tag_id_list:
        queryset = Tag.objects.filter(pk__in=tag_id_list)

    if not queryset:
        messages.error(request, _('Must provide at least one tag.'))
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    try:
        Permission.check_permissions(request.user, (permission_tag_delete,))
    except PermissionDenied:
        queryset = AccessControlList.objects.filter_by_access(
            permission_tag_delete, request.user, queryset
        )

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    if request.method == 'POST':
        for tag in queryset:
            try:
                tag.delete()
                messages.success(
                    request, _('Tag "%s" deleted successfully.') % tag
                )
            except Exception as exception:
                messages.error(
                    request, _('Error deleting tag "%(tag)s": %(error)s') % {
                        'tag': tag, 'error': exception
                    }
                )

        return HttpResponseRedirect(next)

    context = {
        'delete_view': True,
        'previous': previous,
        'message': _('Will be removed from all documents.'),
        'next': next,
        'title': ungettext(
            'Delete the selected tag?',
            'Delete the selected tags?',
            queryset.count()
        )
    }

    if queryset.count() == 1:
        context['object'] = queryset.first()

    return render_to_response(
        'appearance/generic_confirm.html', context,
        context_instance=RequestContext(request)
    )


def tag_multiple_delete(request):
    return tag_delete(
        request, tag_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )


class TagEditView(SingleObjectEditView):
    fields = ('label', 'color')
    model = Tag
    object_permission = permission_tag_edit
    post_action_redirect = reverse_lazy('tags:tag_list')

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit tag: %s') % self.get_object(),
        }


class TagTaggedItemListView(DocumentListView):
    def get_tag(self):
        return get_object_or_404(Tag, pk=self.kwargs['pk'])

    def get_document_queryset(self):
        return self.get_tag().documents.all()

    def get_extra_context(self):
        return {
            'title': _('Documents with the tag: %s') % self.get_tag(),
            'hide_links': True,
            'object': self.get_tag(),
        }


class DocumentTagListView(TagListView):
    def dispatch(self, request, *args, **kwargs):
        self.document = get_object_or_404(Document, pk=self.kwargs['pk'])

        try:
            Permission.check_permissions(
                request.user, (permission_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_view, request.user, self.document
            )

        return super(
            DocumentTagListView, self
        ).dispatch(request, *args, **kwargs)

    def get_extra_context(self):
        return {
            'hide_link': True,
            'object': self.document,
            'title': _('Tags for document: %s') % self.document,
        }

    def get_tag_queryset(self):
        return self.document.attached_tags().all()


def tag_remove(request, document_id=None, document_id_list=None, tag_id=None, tag_id_list=None):
    if document_id:
        documents = Document.objects.filter(pk=document_id)
    elif document_id_list:
        documents = Document.objects.filter(pk__in=document_id_list)

    if not documents:
        messages.error(
            request, _('Must provide at least one tagged document.')
        )
        return HttpResponseRedirect(
            request.META.get(
                'HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL)
            )
        )

    try:
        Permission.check_permissions(request.user, (permission_tag_remove,))
    except PermissionDenied:
        documents = AccessControlList.objects.filter_by_access(
            permission_tag_remove, request.user, documents
        )

    post_action_redirect = None

    previous = request.POST.get('previous', request.GET.get('previous', request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))
    next = request.POST.get('next', request.GET.get('next', post_action_redirect if post_action_redirect else request.META.get('HTTP_REFERER', reverse(settings.LOGIN_REDIRECT_URL))))

    context = {
        'previous': previous,
        'next': next,
    }

    template = 'appearance/generic_confirm.html'
    if tag_id:
        tags = Tag.objects.filter(pk=tag_id)
    elif tag_id_list:
        tags = Tag.objects.filter(pk__in=tag_id_list)
    else:
        template = 'appearance/generic_form.html'

        if request.method == 'POST':
            form = TagListForm(request.POST, user=request.user)
            if form.is_valid():
                tags = Tag.objects.filter(pk=form.cleaned_data['tag'].pk)
        else:
            if not tag_id and not tag_id_list:
                form = TagListForm(user=request.user)
                tags = Tag.objects.none()

        context['form'] = form
        if len(documents) == 1:
            context['object'] = documents.first()
            context['title'] = _(
                'Remove tag from document: %s.'
            ) % ', '.join([unicode(d) for d in documents])
        elif len(documents) > 1:
            context['title'] = _(
                'Remove tag from documents: %s.'
            ) % ', '.join([unicode(d) for d in documents])

    if tags:
        if tags.count() == 1:
            if documents.count() == 1:
                context['object'] = documents.first()
                context['title'] = _(
                    'Remove the tag "%(tag)s" from the document: %(document)s?'
                ) % {
                    'tag': ', '.join([unicode(d) for d in tags]),
                    'document': ', '.join([unicode(d) for d in documents])
                }
            else:
                context['title'] = _(
                    'Remove the tag "%(tag)s" from the documents: %(documents)s?'
                ) % {
                    'tag': ', '.join([unicode(d) for d in tags]),
                    'documents': ', '.join([unicode(d) for d in documents])
                }
        elif tags.count() > 1:
            if documents.count() == 1:
                context['object'] = documents.first()
                context['title'] = _(
                    'Remove the tags: %(tags)s from the document: %(document)s?'
                ) % {
                    'tags': ', '.join([unicode(d) for d in tags]),
                    'document': ', '.join([unicode(d) for d in documents])
                }
            else:
                context['title'] = _(
                    'Remove the tags %(tags)s from the documents: %(documents)s?'
                ) % {
                    'tags': ', '.join([unicode(d) for d in tags]),
                    'documents': ', '.join([unicode(d) for d in documents])
                }

    if request.method == 'POST':
        for document in documents:
            for tag in tags:
                if tag not in document.attached_tags().all():
                    messages.warning(
                        request, _(
                            'Document "%(document)s" wasn\'t tagged as "%(tag)s"'
                        ) % {
                            'document': document, 'tag': tag
                        }
                    )
                else:
                    tag.documents.remove(document)
                    messages.success(
                        request, _(
                            'Tag "%(tag)s" removed successfully from document "%(document)s".'
                        ) % {
                            'document': document, 'tag': tag
                        }
                    )

        return HttpResponseRedirect(next)
    else:
        return render_to_response(
            template, context, context_instance=RequestContext(request)
        )


def single_document_multiple_tag_remove(request, document_id):
    return tag_remove(
        request, document_id=document_id,
        tag_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )


def multiple_documents_selection_tag_remove(request):
    return tag_remove(
        request, document_id_list=request.GET.get(
            'id_list', request.POST.get('id_list', '')
        ).split(',')
    )



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from actstream import action


class Event(object):
    _labels = {}

    @classmethod
    def get_label(cls, name):
        try:
            return cls._labels[name]
        except KeyError:
            return _('Unknown or obsolete event type: {0}'.format(name))

    def __init__(self, name, label):
        self.name = name
        self.label = label
        self.event_type = None
        self.__class__._labels[name] = label

    def commit(self, actor=None, action_object=None, target=None):
        model = apps.get_model('events', 'EventType')

        if not self.event_type:
            self.event_type, created = model.objects.get_or_create(
                name=self.name
            )

        action.send(
            actor or target, actor=actor, verb=self.name,
            action_object=action_object, target=target
        )



from __future__ import absolute_import, unicode_literals

from datetime import date
import logging
import os
import shutil
import tempfile

import gnupg

from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .exceptions import NeedPassphrase, PassphraseError
from .literals import (
    ERROR_MSG_NEED_PASSPHRASE, ERROR_MSG_BAD_PASSPHRASE,
    ERROR_MSG_GOOD_PASSPHRASE, KEY_TYPE_CHOICES, KEY_TYPE_SECRET,
    OUTPUT_MESSAGE_CONTAINS_PRIVATE_KEY
)
from .managers import KeyManager
from .settings import setting_gpg_path

logger = logging.getLogger(__name__)


def gpg_command(function):
    temporary_directory = tempfile.mkdtemp()
    os.chmod(temporary_directory, 0x1C0)

    gpg = gnupg.GPG(
        gnupghome=temporary_directory, gpgbinary=setting_gpg_path.value
    )

    result = function(gpg=gpg)

    shutil.rmtree(temporary_directory)

    return result


@python_2_unicode_compatible
class Key(models.Model):
    key_data = models.TextField(
        help_text=_('ASCII armored version of the key.'),
        verbose_name=_('Key data')
    )
    creation_date = models.DateField(
        editable=False, verbose_name=_('Creation date')
    )
    expiration_date = models.DateField(
        blank=True, editable=False, null=True,
        verbose_name=_('Expiration date')
    )
    fingerprint = models.CharField(
        editable=False, max_length=40, unique=True,
        verbose_name=_('Fingerprint')
    )
    length = models.PositiveIntegerField(
        editable=False, verbose_name=_('Length')
    )
    algorithm = models.PositiveIntegerField(
        editable=False, verbose_name=_('Algorithm')
    )
    user_id = models.TextField(editable=False, verbose_name=_('User ID'))
    key_type = models.CharField(
        choices=KEY_TYPE_CHOICES, editable=False, max_length=3,
        verbose_name=_('Type')
    )

    objects = KeyManager()

    class Meta:
        verbose_name = _('Key')
        verbose_name_plural = _('Keys')

    def clean(self):
        def import_key(gpg):
            return gpg.import_keys(key_data=self.key_data)

        import_results = gpg_command(function=import_key)

        if not import_results.count:
            raise ValidationError(_('Invalid key data'))

        if Key.objects.filter(fingerprint=import_results.fingerprints[0]).exists():
            raise ValidationError(_('Key already exists.'))

    def get_absolute_url(self):
        return reverse('django_gpg:key_detail', args=(self.pk,))

    def save(self, *args, **kwargs):
        temporary_directory = tempfile.mkdtemp()

        os.chmod(temporary_directory, 0x1C0)

        gpg = gnupg.GPG(
            gnupghome=temporary_directory, gpgbinary=setting_gpg_path.value
        )

        import_results = gpg.import_keys(key_data=self.key_data)

        key_info = gpg.list_keys(keys=import_results.fingerprints[0])[0]

        logger.debug('key_info: %s', key_info)

        shutil.rmtree(temporary_directory)

        self.algorithm = key_info['algo']
        self.creation_date = date.fromtimestamp(int(key_info['date']))
        if key_info['expires']:
            self.expiration_date = date.fromtimestamp(int(key_info['expires']))
        self.fingerprint = key_info['fingerprint']
        self.length = int(key_info['length'])
        self.user_id = key_info['uids'][0]
        if OUTPUT_MESSAGE_CONTAINS_PRIVATE_KEY in import_results.results[0]['text']:
            self.key_type = KEY_TYPE_SECRET
        else:
            self.key_type = key_info['type']

        super(Key, self).save(*args, **kwargs)

    def __str__(self):
        return '{} - {}'.format(self.key_id, self.user_id)

    def sign_file(self, file_object, passphrase=None, clearsign=False, detached=False, binary=False, output=None):
        # WARNING: using clearsign=True and subsequent decryption corrupts the
        # file. Appears to be a problem in python-gnupg or gpg itself.
        # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=55647
        # "The problems differ from run to run and file to
        # file, and appear to be due to random data being inserted in the
        # output data stream."

        temporary_directory = tempfile.mkdtemp()

        os.chmod(temporary_directory, 0x1C0)

        gpg = gnupg.GPG(
            gnupghome=temporary_directory, gpgbinary=setting_gpg_path.value
        )

        import_results = gpg.import_keys(key_data=self.key_data)

        file_sign_results = gpg.sign_file(
            file=file_object, keyid=import_results.fingerprints[0],
            passphrase=passphrase, clearsign=clearsign, detach=detached,
            binary=binary, output=output
        )

        shutil.rmtree(temporary_directory)

        logger.debug('file_sign_results.stderr: %s', file_sign_results.stderr)

        if ERROR_MSG_NEED_PASSPHRASE in file_sign_results.stderr:
            if ERROR_MSG_BAD_PASSPHRASE in file_sign_results.stderr:
                raise PassphraseError
            elif ERROR_MSG_GOOD_PASSPHRASE not in file_sign_results.stderr:
                raise NeedPassphrase

        return file_sign_results

    @property
    def key_id(self):
        return self.fingerprint[-8:]



from __future__ import unicode_literals

import os
import shutil
import tempfile

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase, override_settings

from acls.models import AccessControlList
from common.tests.test_views import GenericViewTestCase
from documents.models import Document, DocumentType, NewVersionBlock
from documents.permissions import permission_document_create
from documents.tests import (
    TEST_DOCUMENT_PATH, TEST_SMALL_DOCUMENT_PATH, TEST_DOCUMENT_DESCRIPTION,
    TEST_DOCUMENT_TYPE
)
from documents.tests.test_views import GenericDocumentViewTestCase
from user_management.tests import (
    TEST_ADMIN_EMAIL, TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME,
    TEST_USER_PASSWORD, TEST_USER_USERNAME
)
from ..links import link_upload_version
from ..literals import SOURCE_CHOICE_WEB_FORM
from ..models import StagingFolderSource, WebFormSource
from ..permissions import permission_staging_file_delete

TEST_SOURCE_LABEL = 'test'
TEST_SOURCE_UNCOMPRESS_N = 'n'
TEST_STAGING_PREVIEW_WIDTH = 640


class DocumentUploadTestCase(GenericDocumentViewTestCase):
    def setUp(self):
        super(DocumentUploadTestCase, self).setUp()
        self.source = WebFormSource.objects.create(
            enabled=True, label=TEST_SOURCE_LABEL,
            uncompress=TEST_SOURCE_UNCOMPRESS_N
        )

        self.document.delete()

    def test_upload_wizard_without_permission(self):
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        with open(TEST_DOCUMENT_PATH) as file_object:
            response = self.client.post(
                reverse(
                    'sources:upload_interactive', args=(self.source.pk,)
                ), data={
                    'source-file': file_object,
                    'document_type_id': self.document_type.pk,
                }
            )

        self.assertEqual(response.status_code, 403)
        self.assertEqual(Document.objects.count(), 0)

    def test_upload_wizard_with_permission(self):
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        self.role.permissions.add(
            permission_document_create.stored_permission
        )

        with open(TEST_DOCUMENT_PATH) as file_object:
            response = self.client.post(
                reverse(
                    'sources:upload_interactive', args=(self.source.pk,)
                ), data={
                    'source-file': file_object,
                    'document_type_id': self.document_type.pk,
                }, follow=True
            )

        self.assertTrue(b'queued' in response.content)
        self.assertEqual(Document.objects.count(), 1)

    def test_upload_wizard_with_document_type_access(self):
        """
        Test uploading of documents by granting the document create
        permssion for the document type to the user
        """

        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        # Create an access control entry giving the role the document
        # create permission for the selected document type.
        acl = AccessControlList.objects.create(
            content_object=self.document_type, role=self.role
        )
        acl.permissions.add(permission_document_create.stored_permission)

        with open(TEST_DOCUMENT_PATH) as file_object:
            response = self.client.post(
                reverse(
                    'sources:upload_interactive', args=(self.source.pk,)
                ), data={
                    'source-file': file_object,
                    'document_type_id': self.document_type.pk,
                }, follow=True
            )

        self.assertTrue(b'queued' in response.content)
        self.assertEqual(Document.objects.count(), 1)


@override_settings(OCR_AUTO_OCR=False)
class DocumentUploadIssueTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()

    def tearDown(self):
        self.document_type.delete()

    def test_issue_25(self):
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        # Create new webform source
        self.client.post(
            reverse(
                'sources:setup_source_create', args=(SOURCE_CHOICE_WEB_FORM,)
            ), {'label': 'test', 'uncompress': 'n', 'enabled': True}
        )
        self.assertEqual(WebFormSource.objects.count(), 1)

        # Upload the test document
        with open(TEST_SMALL_DOCUMENT_PATH) as file_descriptor:
            self.client.post(
                reverse('sources:upload_interactive'),
                {
                    'document-language': 'eng', 'source-file': file_descriptor,
                    'document_type_id': self.document_type.pk
                }
            )
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()
        # Test for issue 25 during creation
        # ** description fields was removed from upload from **
        self.assertEqual(document.description, '')

        # Reset description
        document.description = TEST_DOCUMENT_DESCRIPTION
        document.save()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)

        # Test for issue 25 during editing
        self.client.post(
            reverse('documents:document_edit', args=(document.pk,)),
            {
                'description': TEST_DOCUMENT_DESCRIPTION,
                'language': document.language, 'label': document.label
            }
        )
        # Fetch document again and test description
        document = Document.objects.first()
        self.assertEqual(document.description, TEST_DOCUMENT_DESCRIPTION)


class NewDocumentVersionViewTestCase(GenericDocumentViewTestCase):
    def test_new_version_block(self):
        """
        Gitlab issue #231
        User shown option to upload new version of a document even though it
        is blocked by checkout - v2.0.0b2

        Expected results:
            - Link to upload version view should not resolve
            - Upload version view should reject request
        """

        self.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )

        NewVersionBlock.objects.block(self.document)

        response = self.post(
            'sources:upload_version', args=(self.document.pk,),
            follow=True
        )

        self.assertContains(
            response, text='blocked from uploading',
            status_code=200
        )

        response = self.get(
            'documents:document_version_list', args=(self.document.pk,),
            follow=True
        )

        # Needed by the url view resolver
        response.context.current_app = None
        resolved_link = link_upload_version.resolve(context=response.context)

        self.assertEqual(resolved_link, None)


class StagingFolderTestCase(GenericViewTestCase):
    def setUp(self):
        super(StagingFolderTestCase, self).setUp()
        self.temporary_directory = tempfile.mkdtemp()
        # TODO: remove temp directory after test
        shutil.copy(TEST_SMALL_DOCUMENT_PATH, self.temporary_directory)

        self.filename = os.path.basename(TEST_SMALL_DOCUMENT_PATH)

    def tearDown(self):
        super(StagingFolderTestCase, self).tearDown()
        shutil.rmtree(self.temporary_directory)

    def test_staging_folder_delete_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        staging_folder = StagingFolderSource.objects.create(
            label=TEST_SOURCE_LABEL,
            folder_path=self.temporary_directory,
            preview_width=TEST_STAGING_PREVIEW_WIDTH,
            uncompress=TEST_SOURCE_UNCOMPRESS_N,
        )

        self.assertEqual(len(list(staging_folder.get_files())), 1)

        staging_file = list(staging_folder.get_files())[0]

        response = self.post(
            'sources:staging_file_delete', args=(
                staging_folder.pk, staging_file.encoded_filename
            ), follow=True
        )

        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(list(staging_folder.get_files())), 1)

    def test_staging_folder_delete_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        self.role.permissions.add(
            permission_staging_file_delete.stored_permission
        )

        staging_folder = StagingFolderSource.objects.create(
            label=TEST_SOURCE_LABEL,
            folder_path=self.temporary_directory,
            preview_width=TEST_STAGING_PREVIEW_WIDTH,
            uncompress=TEST_SOURCE_UNCOMPRESS_N,
        )

        self.assertEqual(len(list(staging_folder.get_files())), 1)

        staging_file = list(staging_folder.get_files())[0]

        response = self.post(
            'sources:staging_file_delete', args=(
                staging_folder.pk, staging_file.encoded_filename
            ), follow=True
        )

        self.assertContains(response, 'deleted', status_code=200)
        self.assertEqual(len(list(staging_folder.get_files())), 0)



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _


def create_default_document_type(sender, **kwargs):
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )

    if not DocumentType.objects.count():
        DocumentType.objects.create(label=_('Default'))



from __future__ import unicode_literals

from kombu import Exchange, Queue

from django.utils.translation import ugettext_lazy as _

from mayan.celery import app
from common import MayanAppConfig, menu_object, menu_secondary, menu_tools
from common.classes import Package

from navigation import SourceColumn

from .classes import Statistic, StatisticNamespace
from .links import (
    link_execute, link_namespace_details, link_namespace_list,
    link_statistics, link_view
)
from .tasks import task_execute_statistic  # NOQA - Force registration of task


class StatisticsApp(MayanAppConfig):
    name = 'statistics'
    verbose_name = _('Statistics')

    def ready(self):
        super(StatisticsApp, self).ready()

        Package(label='Chart.js', license_text='''
Copyright (c) 2013-2015 Nick Downie

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
        ''')

        SourceColumn(
            source=Statistic,
            # Translators: Schedule here is a verb, the 'schedule' at which the
            # statistic will be updated
            label=_('Schedule'),
            attribute='schedule',
        )

        app.conf.CELERY_QUEUES.extend(
            (
                Queue(
                    'statistics', Exchange('statistics'),
                    routing_key='statistics', delivery_mode=1
                ),
            )
        )

        app.conf.CELERY_ROUTES.update(
            {
                'statistics.tasks.task_execute_statistic': {
                    'queue': 'statistics'
                },
            }
        )

        menu_object.bind_links(links=(link_execute, link_view), sources=(Statistic,))
        menu_object.bind_links(
            links=(link_namespace_details,), sources=(StatisticNamespace,)
        )
        menu_secondary.bind_links(
            links=(link_namespace_list,),
            sources=(StatisticNamespace, 'statistics:namespace_list')
        )
        menu_tools.bind_links(links=(link_statistics,))



from __future__ import unicode_literals

from django.contrib.auth import get_user_model

from common.tests.test_views import GenericViewTestCase

from ..permissions import (
    permission_user_delete, permission_user_edit, permission_user_view
)

from .literals import (
    TEST_USER_PASSWORD, TEST_USER_PASSWORD_EDITED, TEST_USER_USERNAME
)

TEST_USER_TO_DELETE_USERNAME = 'user_to_delete'


class UserManagementViewTestCase(GenericViewTestCase):
    def test_user_set_password_view_no_permissions(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_set_password', args=(self.user.pk,), data={
                'new_password_1': TEST_USER_PASSWORD_EDITED,
                'new_password_2': TEST_USER_PASSWORD_EDITED
            }
        )

        self.assertEqual(response.status_code, 403)

        self.client.logout()
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD_EDITED
        )

        response = self.get('common:current_user_details')

        self.assertEqual(response.status_code, 302)

    def test_user_set_password_view_with_permissions(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_edit.stored_permission)
        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_set_password', args=(self.user.pk,), data={
                'new_password_1': TEST_USER_PASSWORD_EDITED,
                'new_password_2': TEST_USER_PASSWORD_EDITED
            }, follow=True
        )

        self.assertContains(response, text='Successfull', status_code=200)

        self.client.logout()
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD_EDITED
        )
        response = self.get('common:current_user_details')

        self.assertEqual(response.status_code, 200)

    def test_user_multiple_set_password_view_no_permissions(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_multiple_set_password', data={
                'id_list': self.user.pk,
                'new_password_1': TEST_USER_PASSWORD_EDITED,
                'new_password_2': TEST_USER_PASSWORD_EDITED
            }
        )

        self.assertEqual(response.status_code, 403)

        self.client.logout()
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD_EDITED
        )

        response = self.get('common:current_user_details')

        self.assertEqual(response.status_code, 302)

    def test_user_multiple_set_password_view_with_permissions(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_edit.stored_permission)
        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_multiple_set_password', data={
                'id_list': self.user.pk,
                'new_password_1': TEST_USER_PASSWORD_EDITED,
                'new_password_2': TEST_USER_PASSWORD_EDITED
            }, follow=True
        )

        self.assertContains(response, text='Successfull', status_code=200)

        self.client.logout()
        self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD_EDITED
        )
        response = self.get('common:current_user_details')

        self.assertEqual(response.status_code, 200)

    def test_user_delete_view_no_permissions(self):
        user = get_user_model().objects.create(
            username=TEST_USER_TO_DELETE_USERNAME
        )

        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_delete', args=(user.pk,)
        )

        self.assertEqual(response.status_code, 403)
        self.assertEqual(get_user_model().objects.count(), 3)

    def test_user_delete_view_with_permissions(self):
        user = get_user_model().objects.create(
            username=TEST_USER_TO_DELETE_USERNAME
        )

        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_delete.stored_permission)
        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_delete', args=(user.pk,), follow=True
        )

        self.assertContains(response, text='deleted', status_code=200)
        self.assertEqual(get_user_model().objects.count(), 2)

    def test_user_multiple_delete_view_no_permissions(self):
        user = get_user_model().objects.create(
            username=TEST_USER_TO_DELETE_USERNAME
        )

        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_multiple_delete', data={
                'id_list': user.pk
            }
        )

        self.assertEqual(response.status_code, 403)
        self.assertEqual(get_user_model().objects.count(), 3)

    def test_user_multiple_delete_view_with_permissions(self):
        user = get_user_model().objects.create(
            username=TEST_USER_TO_DELETE_USERNAME
        )

        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(permission_user_delete.stored_permission)
        self.role.permissions.add(permission_user_view.stored_permission)

        response = self.post(
            'user_management:user_multiple_delete', data={
                'id_list': user.pk,
            }, follow=True
        )

        self.assertContains(response, text='deleted', status_code=200)
        self.assertEqual(get_user_model().objects.count(), 2)



from __future__ import unicode_literals

from optparse import make_option

from django import apps
from django.core import management


class Command(management.BaseCommand):
    help = 'Run all configured tests for the project.'

    option_list = management.BaseCommand.option_list + (
        make_option(
            '--nomigrations', action='store_true', dest='nomigrations',
            default=False,
            help='Don\'t use migrations when creating the test database.'
        ),
    )

    def handle(self, *args, **options):
        kwargs = {}
        if options.get('nomigrations'):
            kwargs['nomigrations'] = True

        test_apps = [app.name for app in apps.apps.get_app_configs() if getattr(app, 'test', False)]

        print 'Testing: {}'.format(', '.join(test_apps))

        management.call_command('test', *test_apps, interactive=False, **kwargs)



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from acls.models import AccessControlList
from permissions import Permission

from .models import Tag
from .permissions import permission_tag_attach


class TagSerializer(serializers.HyperlinkedModelSerializer):
    documents = serializers.HyperlinkedIdentityField(
        view_name='rest_api:tag-document-list'
    )
    documents_count = serializers.SerializerMethodField()

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:tag-detail'},
        }
        fields = (
            'color', 'documents', 'documents_count', 'id', 'label', 'url'
        )
        model = Tag

    def get_documents_count(self, instance):
        return instance.documents.count()


class NewTagSerializer(serializers.ModelSerializer):
    class Meta:
        fields = (
            'color', 'label', 'id'
        )
        model = Tag


class NewDocumentTagSerializer(serializers.Serializer):
    tag = serializers.IntegerField(
        help_text=_('Primary key of the tag to be added.')
    )

    def create(self, validated_data):
        try:
            tag = Tag.objects.get(pk=validated_data['tag'])

            try:
                Permission.check_permissions(
                    self.context['request'].user, (permission_tag_attach,)
                )
            except PermissionDenied:
                AccessControlList.objects.check_access(
                    permission_tag_attach, self.context['request'], tag
                )

            tag.documents.add(validated_data['document'])
        except Exception as exception:
            raise ValidationError(exception)

        return {'tag': tag.pk}


class DocumentTagSerializer(TagSerializer):
    remove = serializers.SerializerMethodField()

    def get_remove(self, instance):
        return reverse(
            'rest_api:document-tag', args=(
                self.context['document'].pk, instance.pk,
            ), request=self.context['request'], format=self.context['format']
        )

    class Meta(TagSerializer.Meta):
        fields = TagSerializer.Meta.fields + ('remove',)
        read_only_fields = TagSerializer.Meta.fields



from __future__ import unicode_literals

from rest_framework import serializers

from user_management.serializers import UserSerializer

from .models import RecentSearch


class RecentSearchSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='rest_api:recentsearch-detail'
    )
    user = UserSerializer()

    class Meta:
        fields = ('datetime_created', 'hits', 'query', 'url', 'user')
        model = RecentSearch
        read_only_fields = ('datetime_created', 'hits', 'query', 'user')



from __future__ import unicode_literals

from django import apps
from django.utils.translation import ugettext_lazy as _


class LockManagerApp(apps.AppConfig):
    name = 'lock_manager'
    test = True
    verbose_name = _('Lock manager')



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.utils.html import escape
from django.utils.safestring import mark_safe

from acls.models import AccessControlList
from permissions import Permission

from .permissions import permission_tag_view


def widget_document_tags(document, user):
    """
    A tag widget that displays the tags for the given document
    """
    tags_template = []

    tags = document.attached_tags().all()

    try:
        Permission.check_permissions(user, (permission_tag_view,))
    except PermissionDenied:
        tags = AccessControlList.objects.filter_by_access(
            permission_tag_view, user, tags
        )

    for tag in tags:
        tags_template.append(widget_single_tag(tag))

    return mark_safe(''.join(tags_template))


def widget_single_tag(tag):
    return mark_safe(
        '''
            <span class="label label-tag" style="background: {}">{}</span>
        '''.format(tag.color, escape(tag.label).replace(' ', '&nbsp;'))
    )



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import LogEntryListView

urlpatterns = patterns(
    'mailer.views',
    url(
        r'^(?P<document_id>\d+)/send/link/$', 'send_document_link',
        name='send_document_link'
    ),
    url(
        r'^(?P<document_id>\d+)/send/document/$', 'send_document_link',
        {'as_attachment': True}, name='send_document'
    ),
    url(
        r'^log/$', LogEntryListView.as_view(), name='error_log'
    ),
)



from __future__ import absolute_import, unicode_literals

import datetime

import qsstats


def new_documents_per_month():
    from .models import Document

    qss = qsstats.QuerySetStats(Document.passthrough.all(), 'date_added')

    today = datetime.date.today()
    this_year = datetime.date(year=today.year, month=1, day=1)

    return {
        'series': {
            'Documents': map(lambda x: {x[0].month: x[1]}, qss.time_series(start=this_year, end=today, interval='months'))
        }
    }


def new_document_versions_per_month():
    from .models import DocumentVersion

    qss = qsstats.QuerySetStats(DocumentVersion.objects.all(), 'document__date_added')

    today = datetime.date.today()
    this_year = datetime.date(year=today.year, month=1, day=1)

    return {
        'series': {
            'Versions': map(lambda x: {x[0].month: x[1]}, qss.time_series(start=this_year, end=today, interval='months'))
        }
    }


def new_document_pages_per_month():
    from .models import DocumentPage

    qss = qsstats.QuerySetStats(DocumentPage.objects.all(), 'document_version__document__date_added')

    today = datetime.date.today()
    this_year = datetime.date(year=today.year, month=1, day=1)

    return {
        'series': {
            'Pages': map(lambda x: {x[0].month: x[1]}, qss.time_series(start=this_year, end=today, interval='months'))
        }
    }


def total_document_per_month():
    from .models import Document

    qss = qsstats.QuerySetStats(Document.objects.all(), 'date_added')
    this_year = datetime.date.today().year

    result = []

    for month in range(1, datetime.date.today().month + 1):
        next_month = month + 1

        if next_month == 12:
            next_month = 1
            year = this_year + 1
        else:
            next_month = month + 1
            year = this_year

        result.append({month: qss.until(datetime.date(year, next_month, 1))})

    return {
        'series': {
            'Documents': result
        }
    }


def total_document_version_per_month():
    from .models import DocumentVersion

    qss = qsstats.QuerySetStats(DocumentVersion.objects.all(), 'document__date_added')
    this_year = datetime.date.today().year

    result = []

    for month in range(1, datetime.date.today().month + 1):
        next_month = month + 1

        if next_month == 12:
            next_month = 1
            year = this_year + 1
        else:
            next_month = month + 1
            year = this_year

        result.append({month: qss.until(datetime.date(year, next_month, 1))})

    return {
        'series': {
            'Versions': result
        }
    }


def total_document_page_per_month():
    from .models import DocumentPage

    qss = qsstats.QuerySetStats(DocumentPage.objects.all(), 'document_version__document__date_added')
    this_year = datetime.date.today().year

    result = []

    for month in range(1, datetime.date.today().month + 1):
        next_month = month + 1

        if next_month == 12:
            next_month = 1
            year = this_year + 1
        else:
            next_month = month + 1
            year = this_year

        result.append({month: qss.until(datetime.date(year, next_month, 1))})

    return {
        'series': {
            'Pages': result
        }
    }



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from rest_framework import serializers


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    users_count = serializers.SerializerMethodField()

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:group-detail'}
        }
        fields = ('id', 'name', 'url', 'users_count')
        model = Group

    def get_users_count(self, instance):
        return instance.user_set.count()


class UserSerializer(serializers.HyperlinkedModelSerializer):
    groups = GroupSerializer(many=True)

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:user-detail'}
        }
        fields = (
            'first_name', 'date_joined', 'email', 'groups', 'id', 'is_staff',
            'is_active', 'is_superuser', 'last_login', 'last_name',
            'password', 'url', 'username',
        )
        model = get_user_model()
        read_only_fields = ('last_login', 'date_joined')
        write_only_fields = ('password',)

    def restore_object(self, attrs, instance=None):
        user = super(UserSerializer, self).restore_object(attrs, instance)
        if 'password' in attrs:
            user.set_password(attrs['password'])
        return user



from __future__ import absolute_import, unicode_literals

import logging

from django.utils.translation import ugettext_lazy as _

from common import MayanAppConfig, menu_secondary

from .links import link_logout, link_password_change

logger = logging.getLogger(__name__)


class AuthenticationApp(MayanAppConfig):
    name = 'authentication'
    test = True
    verbose_name = _('Authentication')

    def ready(self):
        super(AuthenticationApp, self).ready()

        menu_secondary.bind_links(
            links=(
                link_password_change, link_logout
            ), position=99, sources=(
                'common:current_user_details', 'common:current_user_edit',
                'common:current_user_locale_profile_details',
                'common:current_user_locale_profile_edit',
                'authentication:password_change_view',
                'common:setup_list', 'common:tools_list'
            )
        )



from __future__ import absolute_import, unicode_literals

from documents.tests.test_views import GenericDocumentViewTestCase
from user_management.tests import (
    TEST_USER_USERNAME, TEST_USER_PASSWORD
)

from ..models import Index
from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit
)

TEST_INDEX_LABEL = 'test label'
TEST_INDEX_EDITED_LABEL = 'test edited label'
TEST_INDEX_SLUG = 'test_slug'


class IndexViewTestCase(GenericDocumentViewTestCase):
    def test_index_create_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        response = self.post(
            'indexing:index_setup_create', data={
                'label': TEST_INDEX_LABEL, 'slug': TEST_INDEX_SLUG
            }
        )

        self.assertEquals(response.status_code, 403)
        self.assertEqual(Index.objects.count(), 0)

    def test_index_create_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_document_indexing_create.stored_permission
        )

        response = self.post(
            'indexing:index_setup_create', data={
                'label': TEST_INDEX_LABEL, 'slug': TEST_INDEX_SLUG
            }, follow=True
        )

        self.assertContains(response, text='created', status_code=200)
        self.assertEqual(Index.objects.count(), 1)
        self.assertEqual(Index.objects.first().label, TEST_INDEX_LABEL)

    def test_index_delete_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        index = Index.objects.create(
            label=TEST_INDEX_LABEL, slug=TEST_INDEX_SLUG
        )

        response = self.post('indexing:index_setup_delete', args=(index.pk,))
        self.assertEqual(response.status_code, 403)
        self.assertEqual(Index.objects.count(), 1)

    def test_index_delete_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_document_indexing_delete.stored_permission
        )

        index = Index.objects.create(
            label=TEST_INDEX_LABEL, slug=TEST_INDEX_SLUG
        )

        response = self.post(
            'indexing:index_setup_delete', args=(index.pk,), follow=True
        )

        self.assertContains(response, text='deleted', status_code=200)
        self.assertEqual(Index.objects.count(), 0)

    def test_index_edit_view_no_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        index = Index.objects.create(
            label=TEST_INDEX_LABEL, slug=TEST_INDEX_SLUG
        )

        response = self.post(
            'indexing:index_setup_edit', args=(index.pk,), data={
                'label': TEST_INDEX_EDITED_LABEL, 'slug': TEST_INDEX_SLUG
            }
        )
        self.assertEqual(response.status_code, 403)
        index = Index.objects.get(pk=index.pk)
        self.assertEqual(index.label, TEST_INDEX_LABEL)

    def test_index_edit_view_with_permission(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

        self.role.permissions.add(
            permission_document_indexing_edit.stored_permission
        )

        index = Index.objects.create(
            label=TEST_INDEX_LABEL, slug=TEST_INDEX_SLUG
        )

        response = self.post(
            'indexing:index_setup_edit', args=(index.pk,), data={
                'label': TEST_INDEX_EDITED_LABEL, 'slug': TEST_INDEX_SLUG
            }, follow=True
        )

        index = Index.objects.get(pk=index.pk)
        self.assertContains(response, text='update', status_code=200)
        self.assertEqual(index.label, TEST_INDEX_EDITED_LABEL)



from __future__ import absolute_import, unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from permissions.classes import Permission
from permissions.models import Role
from permissions.tests import TEST_ROLE

from ..permissions import permission_settings_view

TEST_EMAIL = 'test_user@example.com'
TEST_GROUP = 'test group'
TEST_PASSWORD = 'testuserpassword'
TEST_USERNAME = 'test_user'


class SmartSettingViewPermissionsTestCase(TestCase):
    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username=TEST_USERNAME, email=TEST_EMAIL,
            password=TEST_PASSWORD
        )
        self.group = Group.objects.create(name=TEST_GROUP)
        self.role = Role.objects.create(label=TEST_ROLE)

        self.group.user_set.add(self.user)
        self.role.groups.add(self.group)

        Permission.invalidate_cache()

        self.client = Client()
        self.client.login(
            username=TEST_USERNAME, password=TEST_PASSWORD
        )

    def tearDown(self):
        self.group.delete()
        self.role.delete()
        self.user.delete()

    def test_view_access_denied(self):
        response = self.client.get(reverse('settings:namespace_list'))
        self.assertEqual(response.status_code, 403)

        response = self.client.get(
            reverse('settings:namespace_detail', args=('common',),)
        )
        self.assertEqual(response.status_code, 403)

    def test_view_access_permitted(self):
        self.role.permissions.add(permission_settings_view.stored_permission)

        response = self.client.get(reverse('settings:namespace_list'))
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            reverse('settings:namespace_detail', args=('common',),)
        )
        self.assertEqual(response.status_code, 200)
